import 'dart:io' if(dart.library.html) 'dart:html';

import 'package:flutter/material.dart';
import 'package:flutter_carousel_media_slider/carousel_media.dart';
import 'package:video_player/video_player.dart';

class VideoPlayerHelper extends StatefulWidget {
  const VideoPlayerHelper({
    Key? key,
    required this.carouselMedia,
    required this.onComplete,
    required this.placeholder,
  }) : super(key: key);

  final CarouselMedia carouselMedia;
  final VoidCallback onComplete;
  final Widget placeholder;

  @override
  State<VideoPlayerHelper> createState() => _VideoPlayerHelperState();
}

class _VideoPlayerHelperState extends State<VideoPlayerHelper> {
  VideoPlayerController? _controller;
  VoidCallback? _listener;

  @override
  void initState() {
    super.initState();
    _initController().then((_) {
      _playController();
    });
  }

  VoidCallback _listenerSpawner() {
    return () {
      int dur = _controller!.value.duration.inMilliseconds;
      int pos = _controller!.value.position.inMilliseconds;

      if (dur - pos < 1) {
        _stopController();
        _removeController();
        widget.onComplete.call();
      }
    };
  }

  Future<void> _initController() async {
    VideoPlayerController controller;
    String fileUrl = widget.carouselMedia.mediaUrl ?? '';
    switch(widget.carouselMedia.carouselImageSource){
      case CarouselImageSource.asset:
        controller = VideoPlayerController.asset(fileUrl);
        break;
      case CarouselImageSource.network:
        controller = VideoPlayerController.network(fileUrl);
        break;
      case CarouselImageSource.file:
        controller = VideoPlayerController.file(File(fileUrl));
        break;
    }
    _controller = controller;
    await controller.initialize();
  }

  void _removeController() {
    _controller?.dispose();
    _listener = null;
  }

  void _stopController() {
    _controller?.removeListener(_listener!);
    _controller?.pause();
    _controller?.seekTo(const Duration(milliseconds: 0));
  }

  void _playController() async {
    _listener = _listenerSpawner();
    _controller?.addListener(_listener!);
    await _controller?.play();
    setState(() {});
  }

  @override
  Widget build(BuildContext context) {
    return Center(
      child: _controller!.value.isInitialized
          ? AspectRatio(
              aspectRatio: MediaQuery.of(context).size.aspectRatio,
              child: Center(child: VideoPlayer(_controller!)),
            )
          : widget.placeholder,
    );
  }
}
